@@ -26,6 +26,7 @@
 #include "ts_lua_cached_response.h"
 #include "ts_lua_context.h"
 #include "ts_lua_hook.h"
+#include "ts_lua_vconn.h"
 #include "ts_lua_http.h"
 #include "ts_lua_misc.h"
 #include "ts_lua_log.h"
@@ -57,6 +58,54 @@ ts_lua_update_server_response_hdrp(ts_lua_http_ctx *http_ctx)
   }
 }
 
+void
+ts_lua_clear_http_ctx(ts_lua_http_ctx *http_ctx)
+{
+  if (http_ctx->rri == NULL) {
+    if (http_ctx->client_request_url != NULL) {
+      TSHandleMLocRelease(http_ctx->client_request_bufp, http_ctx->client_request_hdrp, http_ctx->client_request_url);
+      http_ctx->client_request_url = NULL;
+    }
+
+    if (http_ctx->client_request_bufp != NULL) {
+      TSHandleMLocRelease(http_ctx->client_request_bufp, TS_NULL_MLOC, http_ctx->client_request_hdrp);
+      http_ctx->client_request_bufp = NULL;
+      http_ctx->client_request_hdrp = NULL;
+    }
+  }
+
+  if (http_ctx->server_request_url != NULL) {
+    TSHandleMLocRelease(http_ctx->server_request_bufp, http_ctx->server_request_hdrp, http_ctx->server_request_url);
+    http_ctx->server_request_url = NULL;
+  }
+
+  if (http_ctx->server_request_hdrp != NULL) {
+    TSHandleMLocRelease(http_ctx->server_request_bufp, TS_NULL_MLOC, http_ctx->server_request_hdrp);
+    http_ctx->server_request_bufp = NULL;
+    http_ctx->server_request_hdrp = NULL;
+  }
+
+  if (http_ctx->server_response_bufp != NULL) {
+    TSHandleMLocRelease(http_ctx->server_response_bufp, TS_NULL_MLOC, http_ctx->server_response_hdrp);
+    http_ctx->server_response_bufp = NULL;
+    http_ctx->server_response_hdrp = NULL;
+  }
+
+  if (http_ctx->client_response_hdrp != NULL) {
+    TSHandleMLocRelease(http_ctx->client_response_bufp, TS_NULL_MLOC, http_ctx->client_response_hdrp);
+    http_ctx->client_response_bufp = NULL;
+    http_ctx->client_response_hdrp = NULL;
+  }
+
+  if (http_ctx->cached_response_bufp != NULL) {
+    TSMimeHdrDestroy(http_ctx->cached_response_bufp, http_ctx->cached_response_hdrp);
+    TSHandleMLocRelease(http_ctx->cached_response_bufp, TS_NULL_MLOC, http_ctx->cached_response_hdrp);
+    TSMBufferDestroy(http_ctx->cached_response_bufp);
+    http_ctx->cached_response_bufp = NULL;
+    http_ctx->cached_response_hdrp = NULL;
+  }
+}
+
 int
 ts_lua_create_vm(ts_lua_main_ctx *arr, int n)
 {
@@ -497,6 +546,8 @@ ts_lua_inject_ts_api(lua_State *L)
   ts_lua_inject_context_api(L);
   ts_lua_inject_hook_api(L);
 
+  ts_lua_inject_vconn_api(L);
+
   ts_lua_inject_http_api(L);
   ts_lua_inject_intercept_api(L);
   ts_lua_inject_misc_api(L);
@@ -619,6 +670,95 @@ ts_lua_destroy_async_ctx(ts_lua_http_ctx *http_ctx)
   TSfree(http_ctx);
 }
 
+void
+ts_lua_set_vconn_ctx(lua_State *L, ts_lua_vconn_ctx *ctx)
+{
+  lua_pushliteral(L, "__ts_vconn_ctx");
+  lua_pushlightuserdata(L, ctx);
+  lua_rawset(L, LUA_GLOBALSINDEX);
+}
+
+ts_lua_vconn_ctx *
+ts_lua_get_vconn_ctx(lua_State *L)
+{
+  ts_lua_vconn_ctx *ctx;
+
+  lua_pushliteral(L, "__ts_vconn_ctx");
+  lua_rawget(L, LUA_GLOBALSINDEX);
+  ctx = lua_touserdata(L, -1);
+
+  lua_pop(L, 1); // pop the ctx out
+
+  return ctx;
+}
+
+ts_lua_vconn_ctx *
+ts_lua_create_vconn_ctx(ts_lua_main_ctx *main_ctx, ts_lua_instance_conf *conf)
+{
+  ts_lua_vconn_ctx *vconn_ctx;
+
+  vconn_ctx = TSmalloc(sizeof(ts_lua_vconn_ctx));
+  memset(vconn_ctx, 0, sizeof(*vconn_ctx));
+
+  lua_State *L = main_ctx->lua;
+  lua_State *l = lua_newthread(L);
+
+  lua_pushlightuserdata(L, conf);
+  lua_rawget(L, LUA_REGISTRYINDEX);
+
+  /* new globals table for coroutine */
+  lua_newtable(l);
+  lua_pushvalue(l, -1);
+  lua_setfield(l, -2, "_G");
+  lua_newtable(l);
+  lua_xmove(L, l, 1);
+  lua_setfield(l, -2, "__index");
+  lua_setmetatable(l, -2);
+
+  lua_replace(l, LUA_GLOBALSINDEX);
+
+  // init coroutine
+  vconn_ctx->ref = luaL_ref(L, LUA_REGISTRYINDEX);
+
+  vconn_ctx->lua  = l;
+  vconn_ctx->mctx = main_ctx;
+
+  // update thread stats
+  ts_lua_ctx_stats *const stats = main_ctx->stats;
+
+  TSMutexLock(stats->mutexp);
+  ++stats->threads;
+  if (stats->threads_max < stats->threads) {
+    stats->threads_max = stats->threads;
+  }
+  TSMutexUnlock(stats->mutexp);
+
+  vconn_ctx->instance_conf = conf;
+
+  ts_lua_set_vconn_ctx(l, vconn_ctx);
+  ts_lua_create_context_table(l);
+
+  return vconn_ctx;
+}
+
+void
+ts_lua_destroy_vconn_ctx(ts_lua_vconn_ctx *vconn_ctx)
+{
+  // update thread stats
+  ts_lua_main_ctx *const main_ctx = vconn_ctx->mctx;
+  ts_lua_ctx_stats *const stats   = main_ctx->stats;
+
+  TSMutexLock(stats->mutexp);
+  --stats->threads;
+  TSMutexUnlock(stats->mutexp);
+
+  if (vconn_ctx->lua) {
+    luaL_unref(vconn_ctx->lua, LUA_REGISTRYINDEX, vconn_ctx->ref);
+  }
+
+  TSfree(vconn_ctx);
+}
+
 void
 ts_lua_set_http_ctx(lua_State *L, ts_lua_http_ctx *ctx)
 {
@@ -703,6 +843,10 @@ ts_lua_destroy_http_ctx(ts_lua_http_ctx *http_ctx)
   ci = &http_ctx->cinfo;
 
   if (http_ctx->rri == NULL) {
+    if (http_ctx->client_request_url) {
+      TSHandleMLocRelease(http_ctx->client_request_bufp, http_ctx->client_request_hdrp, http_ctx->client_request_url);
+    }
+
     if (http_ctx->client_request_bufp) {
       TSHandleMLocRelease(http_ctx->client_request_bufp, TS_NULL_MLOC, http_ctx->client_request_hdrp);
     }
@@ -901,6 +1045,9 @@ int
 ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
 {
   TSHttpTxn txnp;
+  TSMBuffer bufp;
+  TSMLoc hdr_loc;
+  TSMLoc url_loc;
   int event, ret, rc, n, t;
   lua_State *L;
   ts_lua_http_ctx *http_ctx;
@@ -922,6 +1069,24 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
 
   TSMutexLock(main_ctx->mutexp);
 
+  if (!http_ctx->client_request_bufp) {
+    if (TSHttpTxnClientReqGet(txnp, &bufp, &hdr_loc) == TS_SUCCESS) {
+      http_ctx->client_request_bufp = bufp;
+      http_ctx->client_request_hdrp = hdr_loc;
+
+      if (TSHttpHdrUrlGet(bufp, hdr_loc, &url_loc) == TS_SUCCESS) {
+        http_ctx->client_request_url = url_loc;
+      }
+    }
+  }
+
+  if (!http_ctx->client_request_hdrp) {
+    TSMutexUnlock(main_ctx->mutexp);
+
+    TSHttpTxnReenable(txnp, TS_EVENT_HTTP_CONTINUE);
+    return 0;
+  }
+
   ts_lua_set_cont_info(L, ci);
 
   switch (event) {
@@ -933,6 +1098,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_CACHE_LOOKUP_COMPLETE:
@@ -943,6 +1110,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_SEND_REQUEST_HDR:
@@ -953,6 +1122,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_READ_RESPONSE_HDR:
@@ -964,30 +1135,19 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_SEND_RESPONSE_HDR:
 
-    // client response can be changed within a transaction
-    // (e.g. due to the follow redirect feature). So, clearing the pointers
-    // to allow API(s) to fetch the pointers again when it re-enters the hook
-    if (http_ctx->client_response_hdrp != NULL) {
-      TSHandleMLocRelease(http_ctx->client_response_bufp, TS_NULL_MLOC, http_ctx->client_response_hdrp);
-      http_ctx->client_response_bufp = NULL;
-      http_ctx->client_response_hdrp = NULL;
-    }
-
     lua_getglobal(L, TS_LUA_FUNCTION_SEND_RESPONSE);
 
     if (lua_type(L, -1) == LUA_TFUNCTION) {
       ret = lua_resume(L, 0);
     }
 
-    if (http_ctx->client_response_hdrp != NULL) {
-      TSHandleMLocRelease(http_ctx->client_response_bufp, TS_NULL_MLOC, http_ctx->client_response_hdrp);
-      http_ctx->client_response_bufp = NULL;
-      http_ctx->client_response_hdrp = NULL;
-    }
+    ts_lua_clear_http_ctx(http_ctx);
 
     break;
 
@@ -998,6 +1158,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_TXN_START:
@@ -1007,6 +1169,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_PRE_REMAP:
@@ -1016,6 +1180,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_OS_DNS:
@@ -1025,6 +1191,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_READ_CACHE_HDR:
@@ -1034,6 +1202,8 @@ ts_lua_http_cont_handler(TSCont contp, TSEvent ev, void *edata)
       ret = lua_resume(L, 0);
     }
 
+    ts_lua_clear_http_ctx(http_ctx);
+
     break;
 
   case TS_EVENT_HTTP_TXN_CLOSE:
