@@ -153,8 +153,9 @@ ChunkedHandler::read_size()
           }
         } else {
           // We are done parsing size
-          if (num_digits == 0 || running_sum < 0) {
-            // Bogus chunk size
+          if ((num_digits == 0 || running_sum < 0) ||       /* Bogus chunk size */
+              (!ParseRules::is_wslfcr(*tmp) && *tmp != ';') /* Unexpected character */
+          ) {
             state = CHUNK_READ_ERROR;
             done  = true;
             break;
@@ -171,10 +172,16 @@ ChunkedHandler::read_size()
           break;
         }
       } else if (state == CHUNK_READ_SIZE_START) {
-        if (ParseRules::is_lf(*tmp)) {
+        if (ParseRules::is_cr(*tmp)) {
+          // Skip it
+        } else if (ParseRules::is_lf(*tmp) &&
+                   bytes_used <= 2) { // bytes_used should be 2 if it's CRLF, but permit a single LF as well
           running_sum = 0;
           num_digits  = 0;
           state       = CHUNK_READ_SIZE;
+        } else { // Unexpected character
+          state = CHUNK_READ_ERROR;
+          done  = true;
         }
       }
       tmp++;
@@ -1715,7 +1722,7 @@ HttpTunnel::internal_error()
 void
 HttpTunnel::mark_tls_tunnel_active()
 {
-  _tls_tunnel_last_update = Thread::get_hrtime();
+  _tls_tunnel_last_update = ink_get_hrtime();
 
   if (_tls_tunnel_active) {
     return;
@@ -1766,7 +1773,7 @@ HttpTunnel::_is_tls_tunnel_active() const
   // This should not be called if period is 0
   ink_release_assert(period > 0);
 
-  ink_hrtime now = Thread::get_hrtime();
+  ink_hrtime now = ink_get_hrtime();
 
   Debug("http_tunnel", "now=%" PRId64 " last_update=%" PRId64, now, _tls_tunnel_last_update);
 
