@@ -20,26 +20,25 @@
     See the License for the specific language governing permissions and
     limitations under the License.
 */
-#ifndef _URL_MAPPING_PATH_INDEX_H
-#define _URL_MAPPING_PATH_INDEX_H
+#pragma once
+
+#include "tscore/ink_platform.h"
 
-#include "ts/ink_platform.h"
-#undef std // FIXME: remove dependancy on the STL
 #include <map>
 
 #include "URL.h"
 #include "UrlMapping.h"
-#include "ts/Trie.h"
+#include "tscore/Trie.h"
 
 class UrlMappingPathIndex
 {
 public:
   UrlMappingPathIndex() {}
-
   virtual ~UrlMappingPathIndex();
   bool Insert(url_mapping *mapping);
   url_mapping *Search(URL *request_url, int request_port, bool normal_search = true) const;
-  void Print();
+  void Print() const;
+  std::string PrintUrlMappingPathIndex() const;
 
 private:
   typedef Trie<url_mapping> UrlMappingTrie;
@@ -49,8 +48,8 @@ private:
     int port;
 
     UrlMappingTrieKey(int idx, int p) : scheme_wks_idx(idx), port(p) {}
-
-    bool operator<(const UrlMappingTrieKey &rhs) const
+    bool
+    operator<(const UrlMappingTrieKey &rhs) const
     {
       if (scheme_wks_idx == rhs.scheme_wks_idx) {
         return (port < rhs.port);
@@ -65,7 +64,11 @@ private:
   // make copy-constructor and assignment operator private
   // till we properly implement them
   UrlMappingPathIndex(const UrlMappingPathIndex & /* rhs ATS_UNUSED */){};
-  UrlMappingPathIndex &operator=(const UrlMappingPathIndex & /* rhs ATS_UNUSED */) { return *this; }
+  UrlMappingPathIndex &
+  operator=(const UrlMappingPathIndex & /* rhs ATS_UNUSED */)
+  {
+    return *this;
+  }
 
   inline UrlMappingTrie *
   _GetTrie(URL *url, int &idx, int port, bool search = true) const
@@ -90,8 +93,6 @@ private:
     if (group_iter != m_tries.end()) {
       return group_iter->second;
     }
-    return 0;
+    return nullptr;
   }
 };
-
-#endif // _URL_MAPPING_PATH_INDEX_H
